@@ -61,25 +61,61 @@ rndr_escape(const struct latex *st, struct lowdown_buf *ob,
 	return lowdown_latex_esc(ob, dat->data, dat->size);
 }
 
+/*
+ * Escape a URL with a sized buffer.  Return FALSE on error (memory),
+ * TRUE on success. XXX: this is the same function found in nroff.c.
+ */
+static int
+rndr_url_buf(struct lowdown_buf *ob, const char *data, size_t sz,
+    const enum halink_type *type)
+{
+	size_t	 	 i = 0;
+	unsigned char	 ch;
+
+	if (type != NULL && *type == HALINK_EMAIL && sz > 7 &&
+	    strncmp(data, "mailto:", 7) == 0)
+		i = 7;
+
+	for ( ; i < sz; i++) {
+		ch = (unsigned char)data[i];
+		if (!isprint(ch) || strchr(" <>\\^`{|}\"", ch) != NULL) {
+			if (!hbuf_printf(ob, "\\%%%.2X", ch))
+				return 0;
+		} else if (!hbuf_putc(ob, ch))
+			return 0;
+	}
+
+	return 1;
+}
+
+/*
+ * Escape a URL.  Return FALSE on error (memory), TRUE on success.
+ */
+static int
+rndr_url(struct lowdown_buf *ob, const struct lowdown_buf *link,
+    const enum halink_type *type)
+{
+
+	return rndr_url_buf(ob, link->data, link->size, type);
+}
+
 static int
 rndr_autolink(const struct latex *st, struct lowdown_buf *ob,
-	const struct rndr_autolink *param)
+    const struct rndr_autolink *param)
 {
 
 	if (param->link.size == 0)
 		return 1;
 	if (!HBUF_PUTSL(ob, "\\url{"))
 		return 0;
-	if (param->type == HALINK_EMAIL && !HBUF_PUTSL(ob, "mailto:"))
-		return 0;
-	if (!rndr_escape(st, ob, &param->link))
+	if (!rndr_url(ob, &param->link, &param->type))
 		return 0;
 	return HBUF_PUTSL(ob, "}");
 }
 
 static int
 rndr_entity(const struct latex *st, struct lowdown_buf *ob,
-	const struct rndr_entity *param)
+    const struct rndr_entity *param)
 {
 	const char	*tex;
 	unsigned char	 texflags;
@@ -99,7 +135,7 @@ rndr_entity(const struct latex *st, struct lowdown_buf *ob,
 
 static int
 rndr_blockcode(struct lowdown_buf *ob,
-	const struct rndr_blockcode *param)
+    const struct rndr_blockcode *param)
 {
 
 	if (ob->size && !HBUF_PUTSL(ob, "\n"))
@@ -127,7 +163,7 @@ rndr_blockcode(struct lowdown_buf *ob,
 
 static int
 rndr_definition_title(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (!HBUF_PUTSL(ob, "\\item ["))
@@ -139,7 +175,7 @@ rndr_definition_title(struct lowdown_buf *ob,
 
 static int
 rndr_definition(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (!HBUF_PUTSL(ob, "\\begin{description}\n"))
@@ -151,7 +187,7 @@ rndr_definition(struct lowdown_buf *ob,
 
 static int
 rndr_blockquote(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (ob->size && !HBUF_PUTSL(ob, "\n"))
@@ -165,7 +201,7 @@ rndr_blockquote(struct lowdown_buf *ob,
 
 static int
 rndr_codespan(const struct latex *st, struct lowdown_buf *ob,
-	const struct rndr_codespan *param)
+    const struct rndr_codespan *param)
 {
 #if 0
 	HBUF_PUTSL(ob, "\\lstinline{");
@@ -181,7 +217,7 @@ rndr_codespan(const struct latex *st, struct lowdown_buf *ob,
 
 static int
 rndr_triple_emphasis(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (!HBUF_PUTSL(ob, "\\textbf{\\emph{"))
@@ -193,7 +229,7 @@ rndr_triple_emphasis(struct lowdown_buf *ob,
 
 static int
 rndr_double_emphasis(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (!HBUF_PUTSL(ob, "\\textbf{"))
@@ -205,7 +241,7 @@ rndr_double_emphasis(struct lowdown_buf *ob,
 
 static int
 rndr_emphasis(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (!HBUF_PUTSL(ob, "\\emph{"))
@@ -217,7 +253,7 @@ rndr_emphasis(struct lowdown_buf *ob,
 
 static int
 rndr_highlight(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 
 	if (!HBUF_PUTSL(ob, "\\underline{"))
@@ -311,8 +347,7 @@ out:
 
 static int
 rndr_link(const struct latex *st, struct lowdown_buf *ob,
-	const struct lowdown_buf *content,
-	const struct rndr_link *param)
+    const struct lowdown_buf *content, const struct rndr_link *param)
 {
 	int	loc;
 
@@ -332,11 +367,10 @@ rndr_link(const struct latex *st, struct lowdown_buf *ob,
 		return 0;
 	else if (!loc && !HBUF_PUTSL(ob, "\\href{"))
 		return 0;
-
-	if (loc && !lowdown_latex_esc
-	    (ob, &param->link.data[1], param->link.size - 1))
+	if (loc && !rndr_url_buf(ob, &param->link.data[1],
+	    param->link.size - 1, NULL))
 		return 0;
-	else if (!loc && !rndr_escape(st, ob, &param->link))
+	else if (!loc && !rndr_url( ob, &param->link, NULL))
 		return 0;
 	if (!HBUF_PUTSL(ob, "}{"))
 		return 0;
@@ -348,9 +382,8 @@ rndr_link(const struct latex *st, struct lowdown_buf *ob,
 }
 
 static int
-rndr_list(struct lowdown_buf *ob,
-	const struct lowdown_buf *content,
-	const struct rndr_list *param)
+rndr_list(struct lowdown_buf *ob, const struct lowdown_buf *content,
+    const struct rndr_list *param)
 {
 	const char	*type;
 
@@ -373,13 +406,12 @@ rndr_list(struct lowdown_buf *ob,
 }
 
 static int
-rndr_listitem(struct lowdown_buf *ob,
-	const struct lowdown_buf *content,
-	const struct rndr_listitem *param)
+rndr_listitem(struct lowdown_buf *ob, const struct lowdown_buf *content,
+    const struct rndr_listitem *param)
 {
 	size_t	 size;
 
-	/* Only emit <li> if we're not a <dl> list. */
+	/* Only emit \item if we're not a definition list. */
 
 	if (!(param->flags & HLIST_FL_DEF)) {
 		if (!HBUF_PUTSL(ob, "\\item"))
@@ -408,7 +440,7 @@ rndr_listitem(struct lowdown_buf *ob,
 
 static int
 rndr_paragraph(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+    const struct lowdown_buf *content)
 {
 	size_t	i = 0;
 
@@ -429,8 +461,7 @@ rndr_paragraph(struct lowdown_buf *ob,
 
 static int
 rndr_raw_block(struct lowdown_buf *ob,
-	const struct rndr_blockhtml *param,
-	const struct latex *st)
+    const struct rndr_blockhtml *param, const struct latex *st)
 {
 	size_t	org = 0, sz = param->text.size;
 
@@ -557,8 +588,7 @@ rndr_raw_html(const struct latex *st, struct lowdown_buf *ob,
 }
 
 static int
-rndr_table(struct lowdown_buf *ob,
-	const struct lowdown_buf *content)
+rndr_table(struct lowdown_buf *ob, const struct lowdown_buf *content)
 {
 
 	/* Open the table in rndr_table_header. */
@@ -572,8 +602,8 @@ rndr_table(struct lowdown_buf *ob,
 
 static int
 rndr_table_header(struct lowdown_buf *ob,
-	const struct lowdown_buf *content, 
-	const struct rndr_table_header *param)
+    const struct lowdown_buf *content, 
+    const struct rndr_table_header *param)
 {
 	size_t	 i;
 	char	 align;
@@ -600,8 +630,8 @@ rndr_table_header(struct lowdown_buf *ob,
 
 static int
 rndr_tablecell(struct lowdown_buf *ob,
-	const struct lowdown_buf *content, 
-	const struct rndr_table_cell *param)
+    const struct lowdown_buf *content, 
+    const struct rndr_table_cell *param)
 {
 
 	if (!hbuf_putb(ob, content))
@@ -613,8 +643,7 @@ rndr_tablecell(struct lowdown_buf *ob,
 
 static int
 rndr_superscript(struct lowdown_buf *ob,
-	const struct lowdown_buf *content,
-	enum lowdown_rndrt type)
+    const struct lowdown_buf *content, enum lowdown_rndrt type)
 {
 	const char	*elem;
 
@@ -635,7 +664,7 @@ rndr_normal_text(const struct latex *st, struct lowdown_buf *ob,
 
 static int
 rndr_footnote_ref(struct lowdown_buf *ob,
-	const struct lowdown_buf *content, struct latex *st)
+    const struct lowdown_buf *content, struct latex *st)
 {
 
 	if (!hbuf_printf(ob, "\\footnote[%zu]{", ++st->footsz))
@@ -646,8 +675,7 @@ rndr_footnote_ref(struct lowdown_buf *ob,
 }
 
 static int
-rndr_math(struct lowdown_buf *ob,
-	const struct rndr_math *param)
+rndr_math(struct lowdown_buf *ob, const struct rndr_math *param)
 {
 
 	if (param->blockmode && !HBUF_PUTSL(ob, "\\["))
@@ -814,9 +842,8 @@ rndr_meta(struct latex *st, const struct lowdown_node *n,
 }
 
 static int
-rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *arg, 
-	const struct lowdown_node *n)
+rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq, void *arg, 
+    const struct lowdown_node *n)
 {
 	struct lowdown_buf		*tmp;
 	struct latex			*st = arg;
@@ -988,20 +1015,26 @@ out:
 }
 
 int
-lowdown_latex_rndr(struct lowdown_buf *ob,
-	void *arg, const struct lowdown_node *n)
+lowdown_latex_rndr(struct lowdown_buf *ob, void *arg,
+    const struct lowdown_node *n)
 {
 	struct latex		*st = arg;
 	struct lowdown_metaq	 metaq;
 	int			 rc;
 
+	/* Reset header identifiers, metadata, and footnotes. */
+
 	TAILQ_INIT(&st->headers_used);
 	TAILQ_INIT(&metaq);
 	st->headers_offs = 1;
 	st->footsz = 0;
 
+	/* Actually perform rendering. */
+
 	rc = rndr(ob, &metaq, st, n);
 
+	/* Clean up header identifiers and metadata. */
+
 	lowdown_metaq_free(&metaq);
 	hentryq_clear(&st->headers_used);
 	return rc;
